from manga_py.provider import Provider
from .helpers.std import Std


class Manhwa18Net(Provider, Std):

    def get_chapter_index(self) -> str:
        chapter = self.re.search(r'-chapter-(\d+(?:\.\d+)?)', self.chapter)
        return chapter.group(1).replace('.', '-')

    def get_main_content(self):
        return self._get_content('{}/manga-{}.html')

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga-(.+)\.html')

    def get_chapters(self) -> list:
        return self._elements('a.chapter')

    def get_files(self) -> list:
        content = self.http_get(self.chapter)
        parser = self.document_fromstring(content)
        return self._images_helper(parser, '.chapter-img', 'data-original', 'src')


main = Manhwa18Net



from manga_py.provider import Provider
from .helpers.std import Std


class PornComixInfo(Provider, Std):

    def get_archive_name(self) -> str:
        return 'archive'

    def get_chapter_index(self) -> str:
        return '0'

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        return [b'']

    def get_files(self):
        items = self._elements('.gallery-item a > img')
        images = []
        re = self.re.compile(r'(.+/images/.+\d)-\d+x\d+(\.\w{2,7})')
        for i in items:
            g = re.search(i.get('data-lazy-src')).groups()
            images.append('{}{}'.format(*g))
        return images

    def get_cover(self) -> str:
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.get_url()


main = PornComixInfo



from manga_py.provider import Provider
from .helpers import tapas_io
from .helpers.std import Std


class TapasIo(Provider, Std):  # TODO: Login\Password
    helper = None

    def get_archive_name(self) -> str:
        ch = self.chapter
        return self.normal_arc_name([
            ch['scene'],
            ch['title']
        ])

    def get_chapter_index(self) -> str:
        return str(self.chapter['scene'])

    def get_main_content(self):
        content = self._storage.get('main_content', False)
        return content if content else self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self.re.search(r'seriesTitle\s*:\s*\'(.+)\',', self.content).group(1)

    def get_chapters(self):
        items = self.re.search(r'episodeList\s*:\s*(\[.+\]),', self.content).group(1)
        return [i for i in self.json.loads(items)[::-1] if not i['locked']]

    def get_files(self):
        return self.helper.parse_chapter_content()

    def get_cover(self) -> str:
        return self._cover_from_content('#series-thumb img')

    def prepare_cookies(self):
        self.helper = tapas_io.TapasIo(self)

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.helper.chapter_url()


main = TapasIo



from manga_py.provider import Provider
from .helpers.std import Std


class ComicNnNet(Provider, Std):

    def get_archive_name(self) -> str:
        return self.get_chapter_index()

    def get_chapter_index(self) -> str:  # todo
        re = self.re.compile('/truyen-tranh-online/[^/]+/([^/]+)')
        return re.search(self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/truyen-tranh-online/{}')

    def _iframe_hook(self, url):
        content = self.html_fromstring(url)
        iframe = content.cssselect('iframe')
        if iframe:
            url = iframe[0].get('src')
            self.log('Iframe!\n' + url)
        return self.html_fromstring(url)

    def get_manga_name(self) -> str:
        name = self._get_name(r'/truyen-tranh-online/([^/]+)')
        if self.re.search('.+-\d+', name):
            return name
        a = self._iframe_hook(self.get_url())
        self._params['url'] = a.cssselect('.sub-bor h1 a')[0].get('href')
        return self.get_manga_name()

    def get_chapters(self):
        return self._elements('.manga-chapter-head + ul li > a')

    def get_files(self):
        content = self._iframe_hook(self.chapter)
        files = content.cssselect('textarea#txtarea img')
        if files:
            n = self.http().normalize_uri
            return [n(i.get('src')) for i in files]
        return []

    def prepare_cookies(self):
        self._base_cookies()

    def get_cover(self) -> str:
        return self._cover_from_content('.manga-detail .row img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ComicNnNet



from manga_py.provider import Provider
from .helpers.std import Std
from html import escape


class MangaDexOrg(Provider, Std):
    __content = None
    __chapters = None
    __languages = None
    __countries = {
        '': 'Other',
        'bd': 'Bengali',
        'bg': 'Bulgarian',
        'br': 'Portuguese (Br)',
        'cn': 'Chinese (Simp)',
        'ct': 'Catalan',
        'cz': 'Czech',
        'de': 'German',
        'dk': 'Danish',
        'es': 'Spanish (Es)',
        'fi': 'Finnish',
        'fr': 'French',
        'gb': 'English',
        'gr': 'Greek',
        'hk': 'Chinese (Trad)',
        'hu': 'Hungarian',
        'id': 'Indonesian',
        'il': 'Hebrew',
        'in': 'Hindi',
        'ir': 'Persian',
        'it': 'Italian',
        'jp': 'Japanese',
        'kr': 'Korean',
        'lt': 'Lithuanian',
        'mm': 'Burmese',
        'mn': 'Mongolian',
        'mx': 'Spanish (LATAM)',
        'my': 'Malay',
        'nl': 'Dutch',
        'no': 'Norwegian',
        'ph': 'Filipino',
        'pl': 'Polish',
        'pt': 'Portuguese (Pt)',
        'ro': 'Romanian',
        'rs': 'Serbo-Croatian',
        'ru': 'Russian',
        'sa': 'Arabic',
        'se': 'Swedish',
        'th': 'Thai',
        'tr': 'Turkish',
        'ua': 'Ukrainian',
        'vn': 'Vietnamese',
    }

    def get_archive_name(self) -> str:
        prev = super().get_archive_name()
        code = self.chapter['lang_code']
        return '{}-{}'.format(prev, self.__countries.get(code, 'Other'))

    def get_chapter_index(self) -> str:
        return self.chapter['chapter'].replace('.', '-')

    def manga_idx(self):
        return self.re.search(r'/(?:manga|title)/(\d+)', self.get_url()).group(1)

    def get_main_content(self):
        if self.__content is None:
            content = self.http_get('{}/api/?id={}&type=manga'.format(self.domain, self.manga_idx()))
            self.__content = self.json.loads(content)
        return self.__content

    def get_manga_name(self) -> str:
        return self.content['manga']['title']

    @property
    def _chapters(self):
        chapters = []
        for idx in self.content['chapter']:
            ch = self.content['chapter'][idx]  # type: dict
            ch.update({
                'key': idx,
            })
            chapters.append(ch)
        return chapters

    def _quest_languages(self):
        _languages = self._languages
        arg_language = self.arg('language')
        if arg_language is None:
            languages = self.quest(
                [],
                'Available languages:\n{}\n\nPlease, select your lang (empty for all, comma for delimiter lang):'.format(
                    '\n'.join(_languages)
                )).split(',')
        else:
            languages = arg_language.split(',')

        return languages

    def get_chapters(self):
        if self.__chapters is None:
            languages = self._quest_languages()
            _ch = self.filter_chapters(self._chapters, languages)

            translator = self.arg('translator')
            if translator is not None:
                _ch = self.filter_chapters_translator(_ch, translator)

            self.__chapters = _ch

        return self.__chapters

    @property
    def _languages(self) -> list:
        if self.__languages is None:
            self.__languages = []
            self.__fill_languages()
        return self.__languages

    def __fill_languages(self):
        for lang in self._chapters:
            if lang['lang_code'] not in self.__languages:
                self.__languages.append(lang['lang_code'])

    def filter_chapters(self, chapters, languages: list) -> list:
        if len(languages) == 0 or languages[0].strip(' ') == '':
            return chapters
        return [chapter for chapter in chapters if chapter['lang_code'] in languages]

    def filter_chapters_translator(self, chapters, translator: str) -> list:
        enc_translator = escape(translator)
        return [chapter for chapter in chapters if (
                chapter['group_name'] == translator or chapter['group_name'] == enc_translator
        )]

    def get_files(self):
        content = self.json.loads(self.http_get('{}/api/?id={}&server=null&type=chapter'.format(
            self.domain,
            self.chapter['key']
        )))
        return ['{}{}/{}'.format(content['server'], content['hash'], img) for img in content['page_array']]

    def get_cover(self) -> str:
        return '{}{}'.format(
            self.domain,
            self.content['manga']['cover_url'],
        )

    def chapter_for_json(self) -> str:
        return '{}-{}'.format(self.chapter['volume'] or '0', self.chapter['chapter'])


main = MangaDexOrg



import imghdr
from os import path
from typing import Tuple, Optional

from PIL.Image import Image
from PIL import Image as PilImage, ImageChops


class MangaImage:
    _image = None  # type: Image
    src_path = None  # type: str

    def __init__(self, src_path):
        """
        :param src_path:
        """
        if not path.isfile(src_path):
            raise AttributeError('Image not found')

        self.src_path = src_path
        self._image = PilImage.open(src_path)

    @staticmethod
    def new(mode: str, size: Tuple[int, int]):
        return PilImage.new(mode, size)

    @property
    def image(self) -> Image:
        """
        :rtype: Image
        :return:
        """
        return self._image

    @image.setter
    def image(self, image: Image):
        self._image = image

    def gray(self, dest_path: str):
        """
        :param dest_path:
        :return:
        """
        try:
            image = self.image.convert('LA')
        except (ValueError, OSError):
            image = self.image.convert('L')
        if dest_path is not None:
            image.save(dest_path)
        return image

    def convert(self, dest_path: str, quality: int = 95):
        """
        see http://pillow.readthedocs.io/en/3.4.x/handbook/image-file-formats.html
        :param dest_path:
        :param quality:
        :return:
        """
        self.image.save(dest_path, quality=quality)
        return dest_path

    def crop_manual_with_offsets(self, offsets, dest_path: str):
        """
        :param offsets:
        :param dest_path:
        :return:
        """
        left, upper, right, lower = offsets
        width, height = self.image.size
        image = self.image.crop((
            left,
            upper,
            width - right,
            height - lower
        ))
        image.save(dest_path)

    def crop_manual(self, sizes: tuple, dest_path: str):
        """
        :param sizes: The crop rectangle, as a (left, upper, right, lower)-tuple.
        :param dest_path:
        :return:
        """
        self.image.crop(sizes).save(dest_path)

    def crop_auto(self, dest_path: str):
        """
        :param dest_path:
        :return:
        """
        bg = PilImage.new(
            self.image.mode,
            self.image.size,
            self.image.getpixel((0, 0))
        )
        diff = ImageChops.difference(self.image, bg)
        diff = ImageChops.add(diff, diff, 2.0, -100)
        bbox = diff.getbbox()
        if bbox:
            crop = self.image.crop(bbox)
            if dest_path:
                crop.save(dest_path)

    def close(self):
        self.image is not None and self.image.close()

    @staticmethod
    def real_extension(_path):
        img = imghdr.what(_path)
        if img:
            return '.' + img
        return None

    @staticmethod
    def is_image(_path) -> bool:
        return imghdr.what(_path) is not None



from requests import Session

from manga_py.base_classes.web_driver import get_driver
from manga_py.provider import Provider


class TsuminoCom:
    provider = None

    def __init__(self, provider: Provider):
        self.provider = provider

    def get_cookies(self, url):
        driver = get_driver()
        driver.get(url)
        iframe = driver.find_element(".g-recaptcha iframe")
        src = self.provider.http_get(iframe.get_attribute('src'))
        driver.close()

        g_token = self.provider.html_fromstring(src).cssselect('#recaptcha-token')
        session = Session()
        h = session.post('{}/Read/AuthProcess'.format(self.provider.domain), data={
            'g-recaptcha-response': g_token[0].get('value'),
            'Id': 1,
            'Page': 1,
        })
        session.close()
        return h.cookies



from manga_py.provider import Provider
from .helpers.std import Std


class Hentai2ReadCom(Provider, Std):
    images_cdn = 'https://static.hentaicdn.com/hentai'

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        idx = self.re.search('.+/([^/]+)/', chapter)
        return '-'.join(idx.group(1).split('.'))

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        return self._elements('li .chapter-row')

    def get_files(self):
        content = self.http_get(self.chapter)
        selector = r'\'images\'\s*:\s*(\[.+\])'
        items = self.json.loads(self.re.search(selector, content).group(1))
        return ['{}{}'.format(self.images_cdn, i) for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('.ribbon-primary .border-black-op')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = Hentai2ReadCom



from manga_py.provider import Provider
from .helpers.std import Std
from .helpers._http2 import Http2


class MyMangaListOrg(Provider, Std):

    def get_chapter_index(self) -> str:
        # re = self.re.compile(r'/chapter-[^/]+-(\d+)')
        re = self.re.compile(r'/download/[^/]+?(\d+)')
        return re.search(self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/read-{}')

    def get_manga_name(self) -> str:
        if ~self.get_url().find('/read'):
            re = r'/read-([^/]+)'
        else:
            re = r'/chapter-([^/]+)-\d+'
        return self._get_name(re)

    def get_chapters(self):
        return self._elements('.chapter_info_download a')

    def loop_chapters(self):
        http2 = Http2(self)
        http2.download_archives(self._storage['chapters'])

    def get_files(self):
        return []

    def prepare_cookies(self):
        self.cf_scrape(self.get_url())

    def get_cover(self) -> str:
        return self._cover_from_content('img.manga_info_image')

    def book_meta(self) -> dict:
        pass


main = MyMangaListOrg



from manga_py.provider import Provider
from .helpers.std import Std


class TruyenVnsharingSite(Provider, Std):

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'_(\d+(?:\.\d+)?)[^\d]?')
        ch = self.chapter
        return '-'.join(re.search(ch).group(1).split('.'))

    def get_main_content(self):
        name = self._get_name('/read/([^/]+/[^/]+/[^/]+)')
        url = '{}/index/read/{}'
        return self.http_get(url.format(
            self.domain,
            name
        ))

    def get_manga_name(self) -> str:
        return self._get_name('/read/[^/]+/[^/]+/([^/]+)')

    def get_chapters(self):
        return self._elements('#manga-info-list a.title')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.read_content .br_frame > img')

    def get_cover(self) -> str:
        img = self._elements('.info_ava.manga')
        if img and len(img):
            return self.parse_background(img[0])

    def book_meta(self) -> dict:
        # todo meta
        pass


main = TruyenVnsharingSite



from .ac_qq_com import AcQqComCrypt
from .base_lib import BaseLib
from .kissmanga_com import KissMangaComCrypt
from .mangago_me import MangaGoMe
from .mangarock_com import MangaRockComCrypt
from .manhuagui_com import ManhuaGuiComCrypt
from .puzzle import Puzzle
from .sunday_webry_com import SundayWebryCom



from manga_py.provider import Provider
from .helpers.std import Std


class RawDevArtCom(Provider, Std):
    _chapter_selector = r'/chapter/[^\d]+(\d+(?:\.\d+)?)'

    def get_chapter_index(self) -> str:
        idx = self.re.search(self._chapter_selector, self.chapter)
        return '-'.join(idx.group(1).split('.'))

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.wp-manga-chapter > a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.page-break img.wp-manga-chapter-img')

    def get_cover(self) -> str:
        return self._cover_from_content('.summary_image img.img-responsive')


main = RawDevArtCom



class Http2:
    provider = None
    path_join = None
    is_file = None
    chapters = None
    chapters_count = 0

    def __init__(self, provider):
        from manga_py.fs import path_join, is_file
        self.provider = provider
        self.path_join = path_join
        self.is_file = is_file

    def _get_name(self, idx):
        return self.path_join(
            self.provider._params.get('destination'),
            self.provider._storage['manga_name'],
            '{:0>3}-{}.{}'.format(
                idx, self.provider.get_archive_name(),
                self.provider._archive_type()
            )
        )

    def __download(self, idx, name, url):
        _min, _max = self._min_max_calculate()
        self.provider._info.add_volume(
            self.provider.chapter,
            self.provider.get_archive_path()
        )

        self.provider.progress(self.chapters_count, idx)

        if idx < _min or (idx >= _max > 0) or self.is_file(name):
            return False

        if not self.provider._simulate:
            try:
                self.provider.http().download_file(url, name, idx)
            except Exception as e:
                self.provider._info.set_last_volume_error(e)

    def _min_max_calculate(self):
        _min = self.provider._params.get('skip_volumes', 0)
        _max = self.provider._params.get('max_volumes', 0)
        self.chapters_count = len(self.chapters)
        if _max > 0 or _min > 0:
            if _max < self.chapters_count:
                _max = self.chapters_count - _max
            else:
                _max = 0
            self.chapters_count = self.chapters_count - _min - _max
        if _max > 0 and _min > 0:
            _max += _min - 1
        return _min, _max

    def download_archives(self, chapters=None):
        if chapters is None:
            chapters = self.provider._storage['chapters']
        self.chapters = chapters
        for idx, url in enumerate(chapters):
            self.provider.before_download_chapter()
            self.provider.chapter_id = idx
            name = self._get_name(idx)
            idx, url, name = self.before_download(idx, url, name)
            self.__download(idx, name, url)
            self.after_download(idx, name)

    def before_download(self, idx, url, _path):
        return idx, url, _path

    def after_download(self, idx, _path):
        pass



from manga_py.provider import Provider
from .helpers.std import Std


class ComicExtraCom(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/chapter-(.+)', self.chapter)
        if idx:
            return '{}-{}'.format(self.chapter_id, idx.group(1))
        return str(self.chapter_id)

    def get_main_content(self):
        return self._get_content('{}/comic/{}')

    def get_manga_name(self):
        url = self.get_url()
        test = self.re.search('/comic/([^/]+)', url)
        if test:
            return test.group(1)
        return self.re.search('/([^/]+)/chapter', url).group(1)

    def get_chapters(self):
        return self._elements('#list td a')

    def get_files(self):
        url = self.chapter + '/full'
        items = self.html_fromstring(url, '.chapter-container img.chapter_img')
        return [i.get('src') for i in items]

    def get_cover(self):
        return self._cover_from_content('.movie-image img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ComicExtraCom



from .helpers.std import Std
from .mangashiro_net import MangaShiroNet


class SubaPicsCom(MangaShiroNet, Std):
    alter_re_name = r'\.(?:com|net)/([^/]+)-chapter-\d+'
    chapter_re = r'-chapter-(\d+(?:-\d+)?)'

    def get_cover(self) -> str:
        return self._cover_from_content('.imgdesc > img')

    def get_files(self):
        url = self.chapter
        parser = self.html_fromstring(url)
        items = parser.cssselect('#readerarea img')
        return [i.get('src') for i in items]

    def book_meta(self) -> dict:
        # todo meta
        pass


main = SubaPicsCom



from manga_py.crypt.base_lib import BaseLib
from manga_py.provider import Provider
from .helpers.std import Std


class HoduComicsCom(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name({'vol': [
            self.chapter_id,
            self.get_chapter_index()
        ]})

    def get_chapter_index(self) -> str:
        return self.re.search(r'view/(\d+)', self.chapter).group(1)

    def get_main_content(self):
        content = self._storage.get('main_content', None)
        if content is None:
            self._storage['main_content'] = self.http_get(self.get_url())
        return self._storage['main_content']

    def get_manga_name(self) -> str:
        self.http().referer = self.get_url()
        element = self.document_fromstring(self.content, '[property="og:title"]', 0)
        return element.get('content')

    def get_chapters(self):
        items = self._elements('.episode_list .episode_tr.not_need_pay')
        re = self.re.compile(r'(/webtoon/.+?/\d+)')
        n = self.http().normalize_uri
        if len(items) == 0:
            return []
        return [n(re.search(i.get('onclick')).group(1)) for i in items]

    def get_files(self):
        content = self.http_get(self.chapter)
        images = self.re.search(r'toon_img\s*=\s*[\'"](.+?)[\'"]', content)
        if not images:
            return []
        parser = self.document_fromstring(BaseLib.base64decode(images.group(1)).decode())
        return self._images_helper(parser, 'img')

    def get_cover(self) -> str:
        return self._cover_from_content('.episode_bnr > img')

    def book_meta(self) -> dict:
        pass

    def prepare_cookies(self):
        self.cf_scrape(self.get_url())


main = HoduComicsCom



from argparse import ArgumentParser


def _args_general(args_parser: ArgumentParser, version):  # pragma: no cover
    args = args_parser.add_argument_group('General options')

    args.add_argument(
        'url',
        metavar='URL',
        type=str,
        help=(
            '%(metavar)s, i.e. link from manga, to be downloaded.'
        )
    )

    args.add_argument(
        '-v',
        '--version',
        action='version',
        version=version,
        help=(
            'Show %(prog)s\'s version number and exit.'
        )
    )

    args.add_argument(
        '-n',
        '--name',
        metavar='NAME',
        type=str,
        default='',
        help=(
            'Rename manga, i.e. by %(metavar)s, and its folder to where it will be saved locally.'
        )
    )

    args.add_argument(
        '-d',
        '--destination',
        metavar='PATH',
        type=str,
        default='Manga',
        help=(
            'Destination folder to where the manga will be saved locally. '
            'The path will be `./%(metavar)s/manga_name/`.'
        )
    )

    args.add_argument(
        '-P',
        '--no-progress',
        action='store_true',
        help=(
            'Don\'t show progress bar.'
        )
    )

    args.add_argument(
        '--arguments',
        type=str,
        nargs='*',
        help=(
            '!!!WARNING!!! This option in-debug mode'
            'An arbitrary set of arguments that can be used in the provider. '
            'If you need to transfer login, you can do it here. (--arguments login=my-login password=my-pasword) '
            'If the provider requires a language choice, you can pass it (--arguments language=my-language) '
            'If site have multiple translators groups, manga-py can try download only specified translator: '
            ' (--arguments translator=awesome-translator) '
        )
    )




class AcQqComCrypt:
    _provider = None
    _site_key = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="

    def __init__(self, provider):
        self._provider = provider

    def decode(self, data):
        data = self._provider.re.sub('[^A-Za-z0-9%+/=]', '', data)
        a = ''
        e = 0
        while e < len(data) - 4:
            e += 1
            b = self._site_key.find(data[e])
            e += 1
            d = self._site_key.find(data[e])
            e += 1
            f = self._site_key.find(data[e])
            e += 1
            g = self._site_key.find(data[e])

            b = b << 2 | d >> 4
            d = (d & 15) << 4 | f >> 2
            h = (f & 3) << 6 | g
            a += chr(b)

            if f != 64:
                a += chr(d)
            if g != 64:
                a += chr(h)
        return self._protect(a)

    def _protect(self, data):
        try:
            data = self._provider.re.search('({.+}})', data).group(1)
            return self._provider.json.loads(data)
        except Exception:
            return {}



from .read_powermanga_org import ReadPowerMangaOrg


class ReaderChampionScansCom(ReadPowerMangaOrg):

    def get_chapter_index(self):
        idx = super().get_chapter_index().split('-')
        if idx[0] == '0':
            del idx[0]
        return '-'.join(idx)


main = ReaderChampionScansCom



from ..provider import Provider
from .helpers.std import Std
from manga_py.base_classes.web_driver import get_driver
from manga_py.base_classes.web_driver.web_driver import WebDriver
from time import sleep


class JapScanCom(Provider, Std):
    _chapters_selector = '#chapters_list .chapters_list a'
    driver = None

    def get_chapter_index(self) -> str:
        return self.re.search(r'.+/(\d+(?:\.\d+)?)/', self.chapter).group(1).replace('.', '-')

    def get_main_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga/([^/]+)')

    def get_chapters(self) -> list:
        chapters = self._elements(self._chapters_selector)
        n = self.http().normalize_uri
        return [n(i.get('href')) for i in chapters]

    def get_files(self):
        content = self.http_get(self.chapter)
        pages = self.document_fromstring(content, 'select#pages option')
        n = self.http().normalize_uri
        return [n(i.get('value')) for i in pages]

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        _path, idx, _url = self._save_file_params_helper(url, idx)
        self.driver.get(url)
        image = self._images()

        self.after_file_save(_path, idx)
        self._archive.add_file(_path)

        callable(callback) and callback()

        return _path

    def prepare_cookies(self):
        raise RuntimeError('Provider not worked')

        self._params['max_threads'] = 1
        self.driver = get_driver('chrome')

    def _image(self):
        image = self.driver.find_element('#image')
        sleep(5)
        shadow = self.driver.find_element('cnv-vv')
        canvases_len = self.driver.driver.execute_script('return arguments[0].all_canvas.length;', shadow)
        canvases = []
        for i in range(int(canvases_len)):
            canvases.append(self.driver.driver.execute_script(
                'return arguments[0].all_canvas[arguments[1]].toDataURL("image/png").substring(21);',
                shadow, i
            ))
        return None


main = JapScanCom



from lxml import html

from manga_py.provider import Provider
from .helpers.std import Std


class CdmNetComBr(Provider, Std):

    def get_archive_name(self) -> str:
        url = self.chapter
        idx = self.get_chapter_index()
        if ~url.find('/manga/'):
            return 'vol_{:0>3}'.format(idx)
        if ~url.find('/novel/'):
            return 'novel_{:0>3}'.format(idx)

    def get_chapter_index(self) -> str:
        re = self.re.compile('/titulos/[^/]+/[^/]+/[^/]+/([^/]+)')
        return re.search(self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/titulos/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'/titulos/([^/]+)')

    def get_chapters(self):
        return self._elements('.ui .content .table td > a')

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        if ~url.find('/manga/'):
            return super().save_file(idx, callback, url, in_arc_name)
        if ~url.find('/novel/'):
            _path, idx, _url = self._save_file_params_helper(url, idx)
            _path += '.html'
            element = self.html_fromstring(url, '.novel-chapter', 0)
            with open(_path, 'wb') as f:
                f.write(html.tostring(element))
            callable(callback) and callback()
            self.after_file_save(_path, idx)
            self._archive.add_file(_path, in_arc_name)
            return _path

    def _manga(self):
        file_type = '.jpg'
        content = self.http_get(self.chapter)
        re_suffix = self.re.compile(r'urlSulfix\s*=\s*[\'"](.+)[\'"]\s*;')
        re_images = self.re.compile(r'pages\s*=\s*(\[.+\])\s*;')
        suffix = re_suffix.search(content).group(1)
        images = re_images.search(content).group(1)
        images = self.re.sub("'", '"', images)
        images = self.json.loads(self.re.sub(r'",\]', '"]', images))

        self.log(['{}{}{}'.format(suffix, i, file_type) for i in images])

        return ['{}{}{}'.format(suffix, i, file_type) for i in images]

    def get_files(self):
        if ~self.chapter.find('/manga/'):
            return self._manga()
        if ~self.chapter.find('/novel/'):
            return [self.chapter]
        return []

    def get_cover(self) -> str:
        return self._cover_from_content('.content .description img.image')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = CdmNetComBr



from manga_py.provider import Provider
from .helpers.std import Std


class BlogTruyenCom(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'\.\w{2,7}/c(\d+)/', self.chapter)
        return '{}-{}'.format(self.chapter_id, idx.group(1))

    def get_main_content(self):
        url = self._test_main_url(self.get_url())
        return self.http_get(self.http().normalize_uri(url))

    def _test_main_url(self, url):
        if self._test_url(url, r'/c'):
            selector = '.breadcrumbs a + a'
            url = self.html_fromstring(url, selector, 0).get('href')
        return url

    def get_manga_name(self) -> str:
        url = self._test_main_url(self.get_url())
        return self.re.search(r'/\d+/([^/]+)', url).group(1)

    def get_chapters(self):
        return self._elements('#list-chapters .title > a')

    def get_files(self):
        items = self.html_fromstring(self.chapter, '#content img')
        return [i.get('src') for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('.thumbnail img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = BlogTruyenCom



from sys import stderr

from manga_py.crypt import KissMangaComCrypt
from manga_py.provider import Provider
from .helpers.std import Std


class KissMangaCom(Provider, Std):
    __local_data = {
        'iv': b'a5e8e2e9c2721be0a84ad660c472c1f3',
        'key': b'mshsdf832nsdbash20asdm',
    }

    def get_archive_name(self) -> str:
        return '{:0>3}-{}'.format(
            self.chapter_id + 1,
            self.get_chapter_index()
        )

    def get_chapter_index(self) -> str:
        name = self.re.search(r'/Manga/[^/]+/(.+)\?id=(\d+)', self.chapter)
        return '-'.join(name.groups())

    def get_main_content(self):
        return self._get_content('{}/Manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/Manga/([^/]+)')

    def get_chapters(self):
        chapters = self._elements('.listing td a')
        if not len(chapters):
            self.log('Chapters not found', file=stderr)
        return chapters

    def prepare_cookies(self):
        self._params['rename_pages'] = True
        self.cf_scrape(self.get_url())
        self._storage['cookies']['rco_quality'] = 'hq'
        if not self._params['cf-protect']:
            self.log('CloudFlare protect fail!', file=stderr)

    def __decrypt_images(self, crypt, key, hexes):
        images = []
        for i in hexes:
            try:
                img = crypt.decrypt(self.__local_data['iv'], key, i)
                images.append(img.decode('utf-8', errors='ignore').replace('\x10', '').replace('\x0f', ''))

            except Exception as e:
                pass

        return images

    def __check_key(self, crypt, content):
        # if need change key
        need = self.re.search(r'\["([^"]+)"\].\+chko.?=.?chko', content)
        key = self.__local_data['key']
        if need:
            # need last group
            key += crypt.decode_escape(need.group(1))
        else:
            # if need change key
            need = self.re.findall(r'\["([^"]+)"\].*?chko.*?=.*?chko', content)
            if need:
                key = crypt.decode_escape(need[-1])
        return key

    def get_files(self):
        crypt = KissMangaComCrypt()
        content = self.http_get(self.chapter)
        key = self.__check_key(crypt, content)
        hexes = self.re.findall(r'lstImages.push\(wrapKA\(["\']([^"\']+?)["\']\)', content)
        if not hexes:
            self.log('Images not found!', file=stderr)
            return []
        return self.__decrypt_images(crypt, key, hexes)

    def get_cover(self):
        return self._cover_from_content('.rightBox .barContent img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = KissMangaCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaWindowNet(Provider, Std):
    __url = None

    def get_chapter_index(self) -> str:
        return self.chapter[0].replace('.', '-')

    def get_main_content(self):
        return self.http_get(self.__url)

    def get_manga_name(self) -> str:
        title = self.html_fromstring(self.get_url(), '.item-title > a, .nav-title > a', 0)
        self.__url = self.http().normalize_uri(title.get('href'))
        return title.text_content().strip()

    def get_chapters(self):
        items = self._elements('.chapter-list a.chapt')
        result = []
        re = self.re.compile(r'[Cc]h\.(\d+(?:\.\d+)?)')
        n = self.http().normalize_uri
        for i in items:
            text = i.cssselect('b')[0].text_content()
            if 'deleted' not in text.casefold():
                result.append((
                    re.search(text).group(1),
                    n(i.get('href')),
                ))
        return result

    def get_files(self):
        re = self.re.compile(r'images\s*=\s*({.+});')
        content = self.http_get(self.chapter[1])
        items = self.json.loads(re.search(content).group(1))
        return [items[i] for i in sorted(items, key=lambda i: int(i))]

    def get_cover(self) -> str:
        return self._cover_from_content('.attr-cover > img')

    def book_meta(self) -> dict:
        pass

    def chapter_for_json(self) -> str:
        return self.chapter[1]


main = MangaWindowNet



from manga_py.provider import Provider
from .helpers.std import Std


class ZingBoxMe(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index()
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        return str(self.chapter.get('title', '0'))

    def get_main_content(self):
        idx = self.re.search('/manga/(?:[^/]+/)?(\d+)/', self.get_url())
        data = {
            'url': '/manga/getBookDetail/{}'.format(idx.group(1)),
            'method': 'GET',
            'api': '/mangaheatapi/web',
        }
        return self.http_post(self.domain + '/api', data=data)

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/manga/(?:\d+/)?([^/]+)')

    def get_chapters(self):
        try:
            return self.json.loads(self.content).get('child', [])
        except self.json.JSONDecodeError:
            return []

    def _chapter_url(self):
        idx = self.chapter.get('chapterId', 0)
        return '/manga/getChapterImages/{}'.format(idx)

    def get_files(self):
        _ = {
            'url': self._chapter_url(),
            'method': 'GET',
            'api': '/mangaheatapi/web',
        }
        images = self.http_post(self.domain + '/api', data=_)
        return self.json.loads(images).get('images', [])

    def get_cover(self):
        return self._cover_from_content('.comicImg img')

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self._chapter_url()


main = ZingBoxMe



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import traceback
from atexit import register as atexit_register
from json import dumps
from os import makedirs, path
from shutil import rmtree
from sys import exit, stderr
from logging import error, info, warning, basicConfig, DEBUG, WARN


def catch(fn, *args):
    def _catch():
        try:
            fn(*args)
        except Exception as e:
            error(e)

    return _catch

display, driver = None, None


@atexit_register
def before_shutdown():
    from .fs import get_temp_path
    from .base_classes.web_driver import display, driver

    display and display.stop()
    driver and driver.close()
    temp_dir = get_temp_path()
    path.isdir(temp_dir) and rmtree(temp_dir)


def _init_cli(args, _info):
    from .cli import Cli
    error_lvl = -5
    try:
        _info.start()
        cli_mode = Cli(args, _info)
        cli_mode.start()
        code = 0
    except Exception as e:
        _args = args.parse_args()
        if _args.debug:
            traceback.print_tb(e.__traceback__, error_lvl, file=stderr)
        code = 1
        _info.set_error(e)
    return code


def _run_util(args) -> tuple:
    from .info import Info
    parse_args = args.parse_args()
    _info = Info(parse_args)
    code = _init_cli(args, _info)

    if parse_args.print_json:
        _info = dumps(
            _info.get(),
            indent=2,
            separators=(',', ': '),
            sort_keys=True,
        )
    else:
        _info = []

    return code, _info


def _update_all(args):
    from .fs import get_info
    parse_args = args.parse_args()
    info('Update all')
    multi_info = {}

    dst = parse_args.destination
    json_info = get_info(dst)

    for i in json_info:
        parse_args.manga_name = i['manga_name']
        parse_args.url = i['url']
        code, _info = _run_util(args)
        multi_info[i['directory']] = _info
    parse_args.quiet or (parse_args.print_json and print(multi_info))


@catch
def main():
    from .cli.args import get_cli_arguments
    from .fs import get_temp_path
    from .meta import _version

    if ~_version.find('alpha'):
        warning('Alpha release! There may be errors!')
    print('\n'.join((
        'Please remember that all sites earn on advertising.',
        'Remember to visit them from your browser.',
        'Thanks!\n'
    )), file=stderr)

    temp_path = get_temp_path()
    path.isdir(temp_path) or makedirs(temp_path)

    args = get_cli_arguments()
    parse_args = args.parse_args()

    log_format = '"%(levelname)s:%(pathname)s:%(lineno)s:%(asctime)s:%(message)s"'
    basicConfig(level=(DEBUG if parse_args.debug else WARN), format=log_format)

    try:
        code, _info = _run_util(args)
        parse_args.quiet or (parse_args.print_json and print(_info))
    except KeyboardInterrupt:
        warning('\nUser interrupt')
        code = 1

    exit(code)


if __name__ == '__main__':
    main()



from manga_py.provider import Provider
from .helpers.std import Std


class ZMangaNet(Provider, Std):
    _type = 'capitulo'

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'%s-(\d+(?:-\d+)?)' % self._type)
        return re.search(self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/manga-online/{}/')

    def get_manga_name(self) -> str:
        re = r'\.\w{2,7}(?:/manga-online|/read)?/([^/]+?)(?:-%s[^/]+)?/' % self._type
        return self._get_name(re)

    def get_chapters(self):
        return self._elements('.mangabox_line > a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        items = self._images_helper(parser, 'meta[property="og:image:secure_url"]', 'content')
        if len(items) < 1:
            items = self._images_helper(parser, 'meta[property="og:image"]', 'content')
        return items

    def get_cover(self) -> str:
        # return self._cover_from_content('.cover img')
        pass

    def book_meta(self) -> dict:
        pass


main = ZMangaNet



from manga_py.provider import Provider
from .helpers.std import Std


class MangaKakalotCom(Provider, Std):

    def get_chapter_index(self) -> str:
        re = self.re.search('/chapter_([^/]+)', self.chapter)
        return re.group(1).replace('.', '-', 2)

    def get_main_content(self):
        return self.http_get(self.get_url())

    def __new_url(self):
        from requests import get
        from sys import stderr
        with get(self.get_url()) as req:
            if req.url != self.get_url():
                print('New url: %s' % req.url, file=stderr)

            self._params['url'] = req.url
            self._storage['main_content'] = req.text

    def get_manga_name(self) -> str:
        if ~self.get_url().find('/manga/'):
            self.__new_url()

        return self._get_name(r'/(?:read-|manga/)(\w+)')

    def get_chapters(self):
        return self._elements('.chapter-list span a')

    def get_files(self):
        result = self.html_fromstring(self.chapter, '#vungdoc img')
        return [i.get('src') for i in result]

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaKakalotCom



from time import time

from manga_py.provider import Provider
from .helpers.std import Std


class ComicoCoIdTitles(Provider, Std):
    _url = None

    def get_chapter_index(self) -> str:
        return str(self.chapter.get('id', '0'))

    def _manga_id(self):
        idx = self.re.search(r'/titles/(\d+)', self.get_url())
        return idx.group(1)

    def get_main_content(self):
        self._url = '{}/titles/{}'.format(
            self.domain,
            self._manga_id(),
        )
        return self.http_get(self._url)

    def get_manga_name(self) -> str:
        h2 = self.document_fromstring(self.content, '.con > h2', 0)
        return '{} - {}'.format(
            h2.text_content(),
            self._manga_id()
        )

    @staticmethod
    def __parse_page(content):
        items = []
        for i in content.get('data', {}).get('list', []):
            if i.get('salePolicy', {}).get('isFree', False):
                items.append(i)
        return items

    def get_chapters(self):
        items = []
        for page in range(1, 10):
            content = self.http_get('{}/chapters?page={}&_={}'.format(
                self._url,
                page,
                int(time()),
            ))
            try:
                content = self.json.loads(content)
                if content.get('header', {}).get('resultCode', -1) < 0:
                    break
                items += self.__parse_page(content)
            except Exception:
                break
        return items

    def get_files(self):
        parser = self.html_fromstring('{}/chapters/{}'.format(
            self._url,
            self.chapter.get('id'),
        ), '._view', 0)
        return self._images_helper(parser, '._image')

    def get_cover(self) -> str:
        return self._cover_from_content('.bg_img_small img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ComicoCoIdTitles



from manga_py.provider import Provider
from .helpers.std import Std


class MangaHubIo(Provider, Std):
    _api = 'https://api.mghubcdn.com/graphql'
    _cdn = 'https://img.mghubcdn.com/file/imghub/'

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        return self.re.search(r'/chapter/[^/]+/\w+-([^/]+)', chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/(?:manga|chapter)/([^/]+)')

    def get_chapters(self):
        return self._elements('.list-group .list-group-item > a')

    def get_files(self):
        query = "{chapter(x:mh01,slug:\"%(name)s\",number:%(num)d)" \
                "{id,title,mangaID,number,slug,date,pages,noAd,manga" \
                "{id,title,slug,mainSlug,author,isWebtoon,isYaoi,isPorn,isSoftPorn,unauthFile,isLicensed" \
                "}}}" % {'name': self.manga_name, 'num': self.chapter_id + 1}
        content = self.json.loads(self.http_post(self._api, data={
            "query": query
        }))

        pages = content.get('data', {}).get('chapter', {}).get('pages', '{}')
        pages = self.json.loads(pages)

        images = []
        for p in pages:
            images.append('{}{}'.format(self._cdn, pages[p]))
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('.row > div > img.img-responsive')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaHubIo



from .authrone_com import AuthroneCom
from .helpers.std import Std


class ThreeAsqInfo(AuthroneCom, Std):
    _ch_selector = '.mng_det ul.lst > li > a.lst'

    def get_chapter_index(self) -> str:
        return self.re.search(
            r'\.\w{2,7}/[^/]+/([^/]+)',
            self.chapter
        ).group(1).replace('.', '-')

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_files(self):
        return list(set(super().get_files()))  # remove doubles


main = ThreeAsqInfo



import cloudscraper
from logging import error


def cf_scrape(url):  # pragma: no cover

    scraper = cloudscraper.create_scraper()
    try:
        return scraper.get_tokens(url)
    except Exception as e:
        error(e)
        raise e



from manga_py.provider import Provider
from .helpers.std import Std


class MangaLibMe(Provider, Std):

    def get_chapter_index(self) -> str:
        selector = r'\.\w{2,7}/[^/]+/[^\d]+(\d+)/[^\d]+([^/]+)'
        idx = self.re.search(selector, self.chapter).groups()
        return '-'.join(idx)

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        return self._elements('.chapters-list .chapter-item__name a')

    def get_files(self):
        content = self.http_get(self.chapter)
        images = self.re.search(r'__pg\s*=\s*(\[.+\])', content).group(1)
        info = self.re.search(r'__info\s*=\s*(\{.+\})', content).group(1)
        images = self.json.loads(images)
        info = self.json.loads(info)
        _manga = info['img']['url']
        _s = info['servers']
        _server = _s.get('main', _s.get('secondary'))

        return ['{}{}{}'.format(_server, _manga, i['u']) for i in images]

    def get_cover(self):
        return self._cover_from_content('img.manga__cover')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaLibMe



from logging import info, error, debug

__all__ = ['display', 'get_driver']

display = None
driver = None


def get_driver(browser: str = None):
    global driver
    global display

    if driver is not None:
        return driver

    _assert_selenium()

    display = _init_display()

    if browser is None:
        driver = _auto_browser()
        return driver

    if browser == 'chrome':
        driver = _chrome()
        return driver

    if browser == 'firefox':
        driver = _firefox()
        return driver

    raise RuntimeError('Bad driver type')


def _assert_selenium():
    try:
        from selenium.webdriver import __version__
    except ImportError as e:
        error('Selenium not installed. Please, run "pip install selenium"')
        raise e


def _init_display():
    try:
        from pyvirtualdisplay import Display
        _display = Display(visible=False, size=(1920, 1080))
        _display.start()
    except ImportError:
        _display = None
        info('Use real display. See here: https://github.com/ponty/PyVirtualDisplay/blob/master/README.rst')
    return _display


def _auto_browser():
    try:
        _driver = _chrome()
        debug('Use chrome')
        return _driver
    except Exception as ce:
        try:
            _driver = _firefox()
            debug('Use _firefox')
            return _driver
        except Exception as e:
            error('Browser driver init error', ce, e)
            raise e


def _chrome():
    try:
        from .chrome import ChromeDriver
        return ChromeDriver().init_driver()
    except Exception as e:
        error('Chrome browser driver init error')
        raise e


def _firefox():
    try:
        from .firefox import FirefoxDriver
        return FirefoxDriver().init_driver()
    except Exception as e:
        error('Firefox browser driver init error')
        raise e



from manga_py.provider import Provider
from .helpers.std import Std


class WhiteCloudPavilionCom(Provider, Std):

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/manga/free/manga/[^/]+/([^/]+)')
        return re.search(self.chapter).group(1).replace('.', '-')

    def get_main_content(self):
        return self._get_content('{}/manga/free/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/free/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.chapters .chapter-title-rtl a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        items = self._images_helper(parser, '#all img', 'data-src')
        return [i.strip(' \n\r\t\0') for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('.boxed img.img-responsive')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = WhiteCloudPavilionCom



from urllib.parse import unquote_plus

from manga_py.provider import Provider
from .helpers.std import Std


class MangaCanBlogCom(Provider, Std):
    _home_link = None

    def get_archive_name(self) -> str:
        ch = self.chapter
        idx = self.re.search(r'/.+/.+?(?:-indonesia-)(.+)\.html', ch)
        if not idx:
            idx = self.re.search(r'/.+/(.+)\.html', ch)
        idx = idx.group(1)
        if ~idx.find('-terbaru'):
            idx = idx[:idx.find('-terbaru')]
        return self.normal_arc_name({'vol': [self.chapter_id, idx]})

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_main_content(self):
        return self.http_get(self._home_link)

    @staticmethod
    def _clear_name(a):
        name = a.text_content()
        name = unquote_plus(name.split('|')[0].strip())
        if ~name.find(' Indonesia'):
            name = name[:name.find(' Indonesia')]
        return name

    def get_manga_name(self) -> str:
        url = self.get_url()
        selector = '.navbar a[href*=".html"]'
        content = self.http_get(url)
        a = self.document_fromstring(content)
        is_chapter = a.cssselect(selector)
        if len(is_chapter) < 1:
            selector = '#latestchapters h1'
            a = a.cssselect(selector)
            self._home_link = url
        else:
            a = is_chapter[0]
            self._home_link = a.get('href')
        return self._clear_name(a)

    def get_chapters(self):
        items = self._elements('a.chaptersrec')
        result = []
        for i in items:
            url = i.get('href')
            _ = url.find('-terbaru-1')
            if _:
                url = url[:_] + '-terbaru.html'
            result.append(url)
        return result

    def get_files(self):
        content = self.http_get(self.chapter)
        items = self._elements('#imgholder .picture', content)
        return [i.get('src') for i in items]

    def get_cover(self) -> str:
        # return self._cover_from_content('.cover img')
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaCanBlogCom



from manga_py.provider import Provider
from .helpers.std import Std


class MyReadingMangaInfo(Provider, Std):

    def get_chapter_index(self, no_increment=False) -> str:
        return str(self.chapter_id)

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        v = [self.get_url()]  # current chapter
        parser = self._elements('.pagination > a')
        if not parser:
            parser = self._elements('.entry-content p > a')
        v += parser
        return v[::-1]

    def prepare_cookies(self):
        self.cf_scrape(self.get_url())

    def get_files(self):
        selector = '.entry-content div img,.entry-content p img'
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, selector)

    def get_cover(self):
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MyReadingMangaInfo



from manga_py.provider import Provider
from .helpers.std import Std


class PecintaKomikCom(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.chapter
        idx = self.re.search('/manga/[^/]+/(\d+(?:,\d)?)', idx)
        return '-'.join(idx.group(1).split(','))

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        return self._elements('.post-cnt ul > li > a')

    def get_files(self):
        url = self.chapter + '/full'
        parser = self.html_fromstring(url)
        items = parser.cssselect('td a .picture')
        base = parser.cssselect('base[href]')
        if base:
            base = base[0].get('href')
        else:
            base = url
        n = self.http().normalize_uri
        return [n(i.get('src'), base) for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('img.pecintakomik')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = PecintaKomikCom



from .manganelo_com import MangaNeloCom


class MangaBatCom(MangaNeloCom):
    def get_main_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self.text_content(self.content, 'h1.entry-title')


main = MangaBatCom





def _args_debug(args_parser):  # pragma: no cover
    args = args_parser.add_argument_group('Debug / Simulation options')

    args.add_argument(
        '-h',
        '--help',
        action='help',
        help=(
            'Show this help and exit.'
        )
    )

    args.add_argument(
        '--print-json',
        action='store_true',
        help=(
            'Print information about the results in the JSON format (after completion).'
        )
    )

    args.add_argument(
        '--simulate',
        action='store_true',
        help=(
            'Simulate running %(prog)s, where: '
            '1) do not download files and, '
            '2) do not write anything on disk.'
        )
    )

    args.add_argument(
        '--show-current-chapter-info',
        action='store_true',
        help=(
            'Show current processing chapter info.'
        )
    )

    args.add_argument(
        '--debug',
        action='store_true',
        help=(
            'Debug %(prog)s.'
        )
    )

    args.add_argument(
        '-q',
        '--quiet',
        action='store_true',
        help=(
            'Dont show any messages.'
        )
    )




#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function

from setuptools import setup, find_packages

from manga_py.meta import _version, _downloader_uri, _author, _email, _license

REQUIREMENTS = [
    'cloudscraper~=1.2.33',
    'cssselect~=1.1.0',
    'lxml~=4.5.0',
    'packaging~=20.3',
    'Pillow~=7.1.1',
    'progressbar2~=3.50.1',
    'pycryptodome~=3.9.7',
    'PyExecJS~=1.5.1',
    'requests~=2.23.0',
    'urllib3~=1.25.8',
]


long_description = """
Universal manga downloader.

Please see https://github.com/manga-py/manga-py
"""

release_status = 'Development Status :: 5 - Production/Stable'
if ~_version.find('beta'):
    release_status = 'Development Status :: 4 - Beta'
if ~_version.find('alpha'):
    release_status = 'Development Status :: 3 - Alpha'


setup(
    name='manga_py',
    packages=find_packages(exclude=(
        'tests',
        '.github',
        'Manga',
        'helpers',
        'mypy_cache',
    )),
    include_package_data=True,
    version=_version,
    description='Universal assistant download manga.',
    long_description=long_description,
    author=_author,
    author_email=_email,
    url=_downloader_uri,
    zip_safe=False,
    data_files=[
        ('manga_py/storage', [
            'manga_py/storage/.passwords.json.dist',
            'manga_py/storage/.proxy.txt',
            'manga_py/crypt/aes.js',
            'manga_py/crypt/aes_zp.js',
        ]),
    ],
    download_url='{}/archive/{}.tar.gz'.format(_downloader_uri, _version),
    keywords=['manga-downloader', 'manga', 'manga-py'],
    license=_license,
    classifiers=[  # look here https://pypi.python.org/pypi?%3Aaction=list_classifiers
        release_status,
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Environment :: Console',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
    ],
    python_requires='>=3.5.3',
    install_requires=REQUIREMENTS,
    entry_points={
        'console_scripts': [
            'manga-py = manga_py:main',
        ]
    }
)



from requests import get
from time import sleep
import re


class Std:
    def _elements(self, selector, content=None) -> list:
        if not content:
            content = self.content
        return self.document_fromstring(content, selector)

    def _cover_from_content(self, selector, attr='src') -> str:
        image = self._elements(selector)
        if image is not None and len(image):
            return self.http().normalize_uri(image[0].get(attr))

    @staticmethod
    def _first_select_options(parser, selector, skip_first=True) -> list:
        options = 'option'
        if skip_first:
            options = 'option + option'
        select = parser.cssselect(selector)
        if select:
            return select[0].cssselect(options)
        return []

    @classmethod
    def _images_helper(cls, parser, selector, attr='src', alternative_attr='data-src') -> list:
        image = parser.cssselect(selector)
        images = []
        for i in image:
            src = i.get(attr) or i.get(alternative_attr)
            images.append(src.strip(' \r\n\t\0'))
        return images

    @classmethod
    def _idx_to_x2(cls, idx, default=0) -> list:
        return [
            str(idx[0]),
            str(default if len(idx) < 2 or not idx[1] else idx[1])
        ]

    @staticmethod
    def _join_groups(idx, glue='-') -> str:
        result = []
        for i in idx:
            if i:
                result.append(i)
        return glue.join(result)

    def _get_name(self, selector, url=None) -> str:
        if url is None:
            url = self.get_url()
        return re.search(selector, url).group(1)

    def _get_content(self, tpl, **kwargs) -> str:
        try:
            _kw = kwargs.copy()
            _kw.setdefault('domain', self.domain)
            _kw.setdefault('manga_name', self.manga_name)
            return self.http_get(tpl.format(**_kw))
        except Exception:
            return self.http_get(tpl.format(self.domain, self.manga_name))

    def _base_cookies(self, url=None):
        if url is None:
            url = self.get_url()
        cookies = self.http().get_base_cookies(url)
        self._storage['cookies'] = cookies.get_dict()

    def parse_background(self, image) -> str:
        url = re.search(
            r'background.+?url\([\'"]?([^\s]+?)[\'"]?\)',
            image.get('style')
        )
        return self.http().normalize_uri(url.group(1))

    def text_content(self, content, selector, idx: int = 0, strip: bool = True):
        doc = self.document_fromstring(content, selector)
        if not doc:
            return None
        text = doc[idx].text_content()
        if strip:
            text = text.strip()
        return text

    def _download(self, file_name, url, method):
        # clean file downloader
        now_try_count = 0
        while now_try_count < 5:
            with open(file_name, 'wb') as out_file:
                now_try_count += 1
                response = get(url, timeout=60, allow_redirects=True)
                if response.status_code >= 400:
                    self.http().debug and self.log('ERROR! Code {}\nUrl: {}'.format(
                        response.status_code,
                        url,
                    ))
                    sleep(2)
                    continue
                out_file.write(response.content)
                response.close()
                out_file.close()
                break

    @staticmethod
    def _test_url(url: str, path: str = None) -> bool:
        _path = r'https?://.+?\.\w{2,7}'
        if path is not None:
            _path += path
        _re = re.compile(_path)
        return _re.search(url) is not None



from manga_py.provider import Provider
from .helpers.std import Std


class ReadMsNet(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.re.search('/r/[^/]+/([^/]+)/([^/]+)', self.chapter).groups()
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        idx = self.re.search('/r/[^/]+/[^/]+/([^/]+)', self.chapter)
        return idx.group(1)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/(?:manga|r)/([^/]+)')

    def get_chapters(self):
        return self._elements('.table-striped td > a')

    def get_files(self):
        img_selector = 'img#manga-page'
        parser = self.html_fromstring(self.chapter)
        img = self._images_helper(parser, img_selector)
        images = []
        img and images.append(img)
        pages = parser.cssselect('.btn-reader-page .dropdown-menu li + li a')
        for i in pages:
            parser = self.html_fromstring(self.http().normalize_uri(i.get('href')))
            img = self._images_helper(parser, img_selector)
            img and images.append(img)
        return images

    def get_cover(self):
        pass  # FIXME HOME

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadMsNet



from manga_py.provider import Provider
from .helpers.std import Std


class HakiHomeCom(Provider, Std):

    def get_chapter_index(self) -> str:
        selector = '.+/([^/]+)/'
        idx = self.re.search(selector, self.chapter)
        return idx.group(1)

    def get_main_content(self):
        selector = r'(https?://[^/]+/[^/]+/[^/]+-\d+/)'
        url = self.re.search(selector, self.get_url())
        return self.http_get(url.group(1))

    def get_manga_name(self) -> str:
        url = self.get_url()
        selector = r'\.\w{2,7}/[^/]+/(.+?)-\d+/'
        return self.re.search(selector, url).group(1)

    def get_chapters(self):
        return self._elements('.listing a.readchap')

    def get_files(self):
        img_selector = '#con img'
        n = self.http().normalize_uri
        uri = n(self.chapter)
        parser = self.html_fromstring(uri, '#contentchap', 0)
        pages = self._first_select_options(parser, '#botn span > select[onchange]')
        images = self._images_helper(parser, img_selector)

        for i in pages:
            parser = self.html_fromstring(n(i.get('value')), '#contentchap', 0)
            images += self._images_helper(parser, img_selector)

        return images

    def get_cover(self) -> str:
        return self._cover_from_content('.noidung img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = HakiHomeCom



from .hentaifox_com import HentaiFoxCom


class HentaiHandCom(HentaiFoxCom):
    _idx_re = r'/comic/(\d+)'
    _url_str = '{}/comic/{}/'
    _name_selector = '#info h1'
    _archive_prefix = 'HentaiHand_'

    def get_files(self):
        parser = self.document_fromstring(self.content)
        images = self._images_helper(parser, 'a.gallerythumb > img')
        re = self.re.compile(r'(.+/images/)(\d+)')
        return ['{}full/{}.jpg'.format(*re.search(i).groups()) for i in images]


main = HentaiHandCom



from .rawdevart_com import RawDevArtCom


class ZeroScansCom(RawDevArtCom):
    _chapter_selector = r'/chapter-(\d+(?:[^\d]\d+)?)'

    def get_chapter_index(self) -> str:
        ch = self.chapter
        idx = self.re.search(self._chapter_selector, ch)
        idx = idx.group(1)
        test = self.re.search(r'(\d+)[^\d](\d+)', idx)
        if test:
            return '-'.join(test.groups())
        return idx

    def get_chapters(self):
        items = self._elements('.wp-manga-chapter > a')
        n = self.http().normalize_uri
        return [n(i.get('href')).rstrip('/') + '/?style=list' for i in items]

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.page-break img')


main = ZeroScansCom



from urllib.parse import unquote_plus

from manga_py.provider import Provider
from .helpers.std import Std


class ReadHentaiMangaCom(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        return self.re.search(r'\.\w{2,7}/[^/]+/([^/]+)', self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        return self._elements('ul.lst a.lst')

    def get_files(self):
        content = self.http_get(self.chapter)
        escaped_images = self.re.search(r'_img_lst\s*=.+?unescape\(\'(.+)\'\)', content)
        if escaped_images:
            return self.json.loads(unquote_plus(escaped_images.group(1)))
        return []

    def get_cover(self) -> str:
        return self._cover_from_content('img.cvr')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadHentaiMangaCom



from manga_py.provider import Provider
from .helpers.std import Std


class LhScansCom(Provider, Std):
    def get_chapter_index(self) -> str:
        re = self.re.compile(r'-chapter-(\d+(?:\.\d+)?)')
        return re.search(self.chapter).group(1).replace('.', '-')

    def get_main_content(self):
        return self._get_content('{}/manga-{}.html')

    def get_manga_name(self) -> str:
        return self._get_name(r'(?:read|manga)-(.+?)(?:-chapter-.+)?.html')

    def get_chapters(self):
        return self._elements('a.chapter')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.chapter-content .chapter-img')

    def get_cover(self) -> str:
        return self._cover_from_content('.thumbnail img')

    def book_meta(self) -> dict:
        pass


main = LhScansCom



from math import ceil

from manga_py.provider import Provider
from .helpers.std import Std


class KuMangaCom(Provider, Std):
    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_main_content(self):
        url = self.re.search(r'(.+\.\w{2,7}/manga/\d+)', self.get_url())
        return self.http_get('%s/' % url.group(1))

    def get_manga_name(self) -> str:
        selector = r'pagination\(\d+,\'(.+)\',\'pagination\''
        parser = self.re.search(selector, self.content)
        return parser.group(1).strip()

    def _chapters(self, parser):
        items = parser.cssselect('.table h4.title > a')
        chapters = []
        for i in items:
            c = '{}/{}'.format(self.domain, i.get('href'))
            chapters.append(c.replace('/c/', '/leer/'))
        return chapters

    def _url_helper(self):
        idx = self.re.search(r'\.\w{2,7}/manga/(\d+)', self.get_url())
        return '{}/manga/{}/p/%d/{}'.format(
            self.domain,
            idx.group(1),
            self.manga_name
        )

    def get_chapters(self):
        selector = r'\'pagination\',\d+,(\d+),(\d+)'
        pages = self.re.search(selector, self.content).groups()
        pages = ceil(float(pages[0]) / float(pages[1]))
        chapters = []
        url_path = self._url_helper()
        for i in range(int(pages) - 1):
            parser = self.html_fromstring(url_path % (i + 1))
            chapters += self._chapters(parser)
        return chapters

    def _get_real_url(self, url):
        location = self.http().requests(url=url, method='head')
        return location.headers.get('Location', url)

    def get_files(self):
        r = self.http().get_redirect_url
        selector = r'(\[\{"npage".+\}\])'
        content = self.http_get(self.chapter)
        items = self.json.loads(self.re.search(selector, content))
        return [r(i.get('imgURL')) for i in items.group(1)]

    def get_cover(self) -> str:
        return self._cover_from_content('.container img.img-responsive')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = KuMangaCom



from logging import error
from urllib.parse import urlparse


class ProviderParams:
    @property
    def content(self):
        content = self._storage.get('main_content', None)
        if content is None:
            content = self.get_main_content()
        return content

    @property
    def manga_name(self) -> str:
        name = self._storage.get('manga_name', None)
        if name is None:
            name = self.get_manga_name()
        return name

    @property
    def name(self) -> str:
        name = self._params.get('name', '')
        if not len(name):
            name = self.manga_name
        return name

    @property
    def domain(self) -> str:
        _url = self._params['url']
        try:
            if not self._storage.get('domain_uri', None):
                parsed = urlparse(_url, 'https')
                self._storage['domain_uri'] = '{}://{}'.format(
                    parsed.scheme,
                    parsed.netloc
                )
            return self._storage.get('domain_uri', '')
        except Exception:
            error('url "%s" is broken!' % _url)
            exit()

    @property
    def chapter(self):
        return self._storage['chapters'][self.chapter_id]
